package MT::Plugin::DynamicMTML;
use strict;
use MT;
use MT::Plugin;
use base qw( MT::Plugin );

use MT::Permission;
use MT::Entry;
use MT::Request;

@MT::Plugin::DynamicMTML::ISA = qw( MT::Plugin );

use MT::Util qw( trim );
use PowerCMS::Util qw( get_childlen_files powercms_files_dir powercms_files_dir_path
                       is_user_can is_cms add_slash site_path register_templates_to
                       get_user to_utf8 utf8_on is_ua_keitai is_ua_iPhone );

our $VERSION = '0.8';
our $SCHEMA_VERSION = '0.15';

my $plugin = new MT::Plugin::DynamicMTML( {
    id   => 'DynamicMTML',
    key  => 'dynamicmtml',
    name => 'DynamicMTML',
    description => '<__trans phrase="_PLUGIN_DESCRIPTION">',
    author_name => 'Alfasado Inc.',
    author_link => 'http://alfasado.net/',
    version => $VERSION,
    schema_version => $SCHEMA_VERSION,
    l10n_class => 'DynamicMTML::L10N',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        object_types => {
            blog => {
                'dynamic_cache' => 'boolean',
                'dynamic_conditional' => 'boolean',
                'search_cache' => 'boolean',
                'search_conditional' => 'boolean',
                'search_cache_expiration' => 'integer',
                'dynamic_extension' => 'string(255)',
                'exclude_extension' => 'string(255)',
                'index_files' => 'string(255)',
            },
        },
        applications => {
            cms => {
                menus => {
                    'tools:clear_cache' => {
                        label => 'Flush Dynamic Cache',
                        mode  => 'flush_dynamic_cache',
                        order => 20000,
                        condition  => \&_dynamic_permission,
                    },
                },
                methods => {
                    install_dynamic_mtml =>
                        'MT::Plugin::DynamicMTML::_install_dynamic_mtml',
                    flush_dynamic_cache =>
                        'MT::Plugin::DynamicMTML::_flush_dynamic_cache',
                },
            },
        },
        callbacks => {
            'cms_post_save.blog'
                => \&_post_save_blog,
            'MT::App::CMS::template_source.list_template'
                => \&_list_template_source,
            'MT::App::CMS::template_param.list_template'
                => \&_list_template_param,
            'MT::App::CMS::template_param.cfg_prefs'
                => \&_cfg_prefs_param,
            'MT::App::CMS::template_source.cfg_prefs'
                => \&_cfg_prefs_source,
            'build_file'
                 => \&_build_file,
            'build_dynamic'
                 => \&_build_dynamic,
        },
        tags => {
            block => {
                'RawMTML' => \&_hdlr_raw_mtml,
                'CommentOut' => \&_hdlr_comment_out,
                'CommentStrip' => \&_hdlr_comment_strip,
                'LoginAuthorCtx' => \&_hdlr_login_author_ctx,
                'SearchEntries' => \&_hdlr_search_entries,
                'IfLogin?' => \&_hdlr_if_login,
                'IfKeitai?' => \&_hdlr_if_keitai,
                'IfiPhone?' => \&_hdlr_if_iphone,
            },
            function => {
                'RawMTMLTag' => \&_hdlr_raw_mtml_tag,
                'BlogDynamicCache' => \&_hdlr_dynamic_cache,
                'BlogDynamicConditional' => \&_hdlr_dynamic_conditional,
                'BlogDynamicSearchCache' => \&_hdlr_dynamic_search_cache,
                'BlogDynamicSearchConditional' => \&_hdlr_dynamic_search_conditional,
                'BlogDynamicSearchCacheExpiration' => \&_hdlr_dynamic_search_cache_expiration,
                'BlogDynamicDirectoryIndex' => \&_hdlr_dynamic_directory_index,
                'BlogDynamicExtension' => \&_hdlr_dynamic_extension,
                'BlogDynamicExcludeExtension' => \&_hdlr_dynamic_exclude_extension,
                'BlogFilesMatchDirective' => \&_hdlr_blog_files_match_directive,
                'BlogFilesDirective' => \&_hdlr_blog_files_directive,
                'Query' => \&_hdlr_query,
                'UserAgent' => \&_hdlr_user_agent,
                'PluginPath' => \&_hdlr_plugin_path,
            },
            modifier => {
                'build_mtml' => \&_filter_build_mtml,
            },
        },
   } );
}

sub _dynamic_permission {
    my $blog = shift;
    my $app = MT->instance();
    $blog = $app->blog unless $blog;
    my $user  = $app->user;
    if ( $blog ) {
        if ( $blog->dynamic_cache ) {
            if ( is_user_can( $blog, $user, 'create_post' ) ) {
                return 1;
            }
            if ( is_user_can( $blog, $user, 'edit_templates' ) ) {
                return 1;
            }
        }
    } else {
        # my $count = MT::Blog->count( { class => '*', dynamic_cache => 1 } );
        # return 0 unless $count;
        my %terms1 = ( author_id   => $user->id,
                       permissions => { like => '%create_post%' } );
        my %terms2 = ( author_id   => $user->id,
                       permissions => { like => '%edit_all_posts%' } );
        my %terms3 = ( author_id   => $user->id,
                       permissions => { like => '%administer%' } );
        my %terms4 = ( author_id   => $user->id,
                       permissions => { like => '%edit_templates%' } );
        my $permission = MT::Permission->load( [ \%terms1, '-or',
                                                 \%terms2, '-or',
                                                 \%terms3, '-or',
                                                 \%terms4 ] );
        return 1 if defined $permission;
    }
    return 0;
}

sub _install_dynamic_mtml {
    my $app = shift;
    my $do;
    if ( $app->blog ) {
        unless ( is_user_can( $app->blog, $app->user, 'edit_templates' ) ) {
            return $app->trans_error( 'Permission denied.' );
        }
        require MT::Template;
        my $bootstrapper = MT::Template->load( { blog_id => $app->blog->id,
                                                 outfile => 'mtview.php' } );
        unless ( $bootstrapper ) {
            my %template = (
                dynamic_mtml_bootstrapper => {
                    name => 'DynamicMTML Bootstrapper',
                    path => 'mtview_php.tmpl',
                    type => 'index',
                    outfile => 'mtview.php',
                    rebuild_me => 1,
                    build_type => 1,
                },
            );
            if ( register_templates_to( $app->blog->id, $plugin, \%template ) ) {
                $do = 1;
            }
        }
        my $htacess = MT::Template->load( { blog_id => $app->blog->id,
                                            outfile => '.htacess' } );
        unless ( $htacess ) {
            my %template = (
                dynamic_mtml_htaccess => {
                    name => 'DynamicMTML .htaccess',
                    path => '_htaccess.tmpl',
                    type => 'index',
                    outfile => '.htaccess',
                    rebuild_me => 1,
                    build_type => 1,
                },
            );
            if ( register_templates_to( $app->blog->id, $plugin, \%template ) ) {
                $do = 1;
            }
        }
    }
    if ( $do ) {
        $app->add_return_arg( installed_dynamicmtml => 1 );
    } else {
        $app->add_return_arg( not_installed_dynamicmtml => 1 );
    }
    $app->call_return();
}

sub _flush_dynamic_cache {
    my $app = shift;
    my $do;
    if (! _dynamic_permission() ) {
        return $app->trans_error( 'Permission denied.' );
    }
    my %param;
    $param{ page_title } = $plugin->translate( 'Flush Dynamic Cache' );
    $app->{ plugin_template_path } = File::Spec->catdir( $plugin->path, 'tmpl' );
    my $tmpl = 'system_msg.tmpl';
    my $powercms_files_dir = powercms_files_dir();
    my $statusmsg;
    if (! $powercms_files_dir ) {
        if ( $app->param( 'return_args' ) ) {
            $app->add_return_arg( no_cache_directory => 1 );
            $app->call_return();
        } else {
            $statusmsg = $plugin->translate( 'Files for PowerCMS directory unexists. Please make directory [_1], and give enough permission to write from web server.',
                                             powercms_files_dir_path() );
            $param{ 'statusmsg' } = $statusmsg;
            return $app->build_page( $tmpl, \%param );
        }
    }
    my $cache_dir = File::Spec->catdir( $powercms_files_dir, 'cache' );
    my @blogs;
    if ( $app->blog ) {
        push ( @blogs, $app->blog );
    } else {
        @blogs = MT::Blog->load( { class => '*', dynamic_cache => 1 } );
    }
    for my $blog ( @blogs ) {
        next unless _dynamic_permission( $blog );
        my $search = 'blog_id_' . $blog->id;
        if ( -d $cache_dir ) {
            my @caches = get_childlen_files( $cache_dir, "/$search/" );
            for my $cache ( @caches ) {
                unlink $cache;
                $do = 1;
            }
        }
        my $templates_c = File::Spec->catdir( site_path( $blog ), 'templates_c' ) ;
        if ( -d $templates_c ) {
            my @template = get_childlen_files( $templates_c, "/$search/" );
            for my $tmpl ( @template ) {
                unlink $tmpl;
                $do = 1;
            }
        }
    }
    if ( $app->param( 'return_args' ) ) {
        if ( $do ) {
            $app->add_return_arg( flush_dynamic_cache => 1 );
        } else {
            $app->add_return_arg( not_flush_dynamic_cache => 1 );
        }
        $app->call_return();
    } else {
        if ( $do ) {
            $statusmsg = $plugin->translate( 'Flush Dynamic Cache was successful.' );
        } else {
            $statusmsg = $plugin->translate( 'Cache file was not found.' );
        }
        $param{ 'statusmsg' } = $statusmsg;
    }
    return $app->build_page( $tmpl, \%param );
}

sub _build_file {
    my ( $eh, %args ) = @_;
    my $app = MT->instance();
    my $file  = $args{ File };
    my $blog  = $args{ Blog };
    my $fi    = $args{ FileInfo };
    my $html  = $args{ Content };
    if ( defined $blog ) {
        return 1 unless $blog->dynamic_cache;
        if ( $$html =~ /<\${0,1}mt/i ) {
            require Digest::MD5;
            my $path = Digest::MD5::md5_hex( $file );
            my $cache = 'blog_' . $blog->id . '_' . $path;
            my $powercms_files_dir = powercms_files_dir();
            if ( $powercms_files_dir ) {
                my $cache_dir = File::Spec->catdir( $powercms_files_dir, 'cache' );
                if ( -d $cache_dir ) {
                    my @caches = get_childlen_files( $cache_dir, "/$cache/" );
                    for my $cache ( @caches ) {
                        unlink $cache;
                    }
                }
            }
            my $r = MT::Request->instance;
            my $self = $r->cache( 'clear_template:' . $fi->template_id );
            return 1 if $self;
            my $templates_c = File::Spec->catdir( site_path( $blog ), 'templates_c' ) ;
            if ( -d $templates_c ) {
                my $search = '_mtml_tpl_id_' . $fi->template_id . '.php';
                my @template = get_childlen_files( $templates_c, "/$search/" );
                for my $tmpl ( @template ) {
                    unlink $tmpl;
                }
                $r->cache( 'clear_template:' . $fi->template_id, 1 );
            }
        }
    }
    return 1;
}

sub _build_dynamic {
    my ( $eh, %args ) = @_;
    my $app = MT->instance();
    my $file  = $args{ File };
    my $blog  = $args{ Blog };
    my $fi    = $args{ FileInfo };
    if ( defined $blog ) {
        return 1 unless $blog->dynamic_cache;
        require Digest::MD5;
        my $path = Digest::MD5::md5_hex( $file );
        my $cache = 'blog_' . $blog->id . '_' . $path;
        my $powercms_files_dir = powercms_files_dir();
        return 1 unless $powercms_files_dir;
        my $cache_dir = File::Spec->catdir( $powercms_files_dir, 'cache' );
        if ( -d $cache_dir ) {
            my @caches = get_childlen_files( $cache_dir, "/$cache/" );
            for my $cache ( @caches ) {
                unlink $cache;
            }
        }
    }
    return 1;
}

sub _post_save_blog {
    my ( $eh, $app, $obj, $original ) = @_;
    $obj->dynamic_cache( $app->param( 'dynamic_cache' ) );
    $obj->dynamic_conditional( $app->param( 'dynamic_conditional' ) );
    $obj->save or die $obj->errstr;
}

sub _hdlr_raw_mtml {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $uncompiled = $ctx->stash( 'uncompiled' ) || '';
    if ( is_cms( $app ) ) {
        my $mode = $app->mode;
        if ( $mode !~ /preview/ ) {
            return $uncompiled;
        } else {
            my $build = _hdlr_pass_tokens( $ctx, $args, $cond );
            return _filter_build_mtml( $build, $args, $ctx );
        }
    } else {
        return $uncompiled;
    }
}

sub _hdlr_raw_mtml_tag {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $tag = $args->{ tag };
    my $params = $args->{ params };
    $tag = "<$tag $params>";
    $tag =~ s/\s>$/>/;
    return $tag;
}

sub _hdlr_comment_out {
    my ( $ctx, $args, $cond ) = @_;
    if ( $args->{ invisible } ) {
        return '';
    }
    my $content = _hdlr_pass_tokens( $ctx, $args, $cond );
    $content = '<!--' . $content . '-->';
    return $content;
}

sub _hdlr_comment_strip {
    my ( $ctx, $args, $cond ) = @_;
    my $content = _hdlr_pass_tokens( $ctx, $args, $cond );
    $content = trim( $content );
    my $begin = quotemeta( '<!--' );
    my $end   = quotemeta( '-->' );
    $content =~ s/$begin//g;
    $content =~ s/$end//g;
    return $content;
}

sub _hdlr_login_author_ctx {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $user = get_user( $app );
    return '' unless defined $user;
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    $ctx->stash( 'author', $user ) if $user;
    $ctx->var( 'author_archive', 1 );
    my $out = $builder->build( $ctx, $tokens, $cond );
    return $out if $out;
    return '';
}

sub _hdlr_search_entries {
    my ( $ctx, $args, $cond ) = @_;
    my ( $ctx, $args, $cond ) = @_;
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    my $blog = $ctx->stash( 'blog' );
    my $blog_id = $blog->id;
    my $query = $args->{ query };
    $query = "%$query%";
    my $target = $args->{ target };
    $target = 'text' if ( $target && (! MT::Entry->has_column( $target ) ) );
    my $operator = $args->{ operator };
    if ( $operator ) {
        $operator = lc ( $operator );
    } else {
        $operator = 'like'
    }
    my $lastn = $args->{ lastn };
    $lastn = 100 unless $lastn;
    my $offset = $args->{ offset };
    $offset = 0 unless $offset;
    my $class = $args->{ class };
    $class = 'entry' unless $class;
    my $sort_by = $args->{ sort_by };
    $sort_by = 'authored_on' unless $sort_by;
    my $sort_order = $args->{ sort_order };
    $sort_order = 'descend' unless $sort_order;
    
    my $args = { limit => $lastn,
                  offset => $offset,
                 'sort' => $sort_by,
                  direction => $sort_order };
    my @entries;
    if ( $target ) {
        my $terms = { blog_id => $blog_id, status => MT::Entry::RELEASE(),
                      $target => { $operator => $query } };
        @entries = MT->model( $class )->load( $terms, $args );
    } else {
        my %terms1 = ( blog_id => $blog_id,
                       status  => MT::Entry::RELEASE(),
                       title   => { $operator => $query } );
        my %terms2 = ( blog_id => $blog_id,
                       status  => MT::Entry::RELEASE(),
                       text    => { $operator => $query } );
        my %terms3 = ( blog_id => $blog_id,
                       status  => MT::Entry::RELEASE(),
                       text_more => { $operator => $query } );
        my %terms4 = ( blog_id => $blog_id,
                       status  => MT::Entry::RELEASE(),
                       keywords => { $operator => $query } );
        my %terms5 = ( blog_id => $blog_id,
                       status  => MT::Entry::RELEASE(),
                       excerpt => { $operator => $query } );
        @entries = MT->model( $class )->load( [ \%terms1, '-or',
                                                \%terms2, '-or',
                                                \%terms3, '-or',
                                                \%terms4 ],
                                                 $args );
    }
    my $res = '';
    my $i = 0;
    my $odd = 1; my $even = 0;
    for my $entry ( @entries ) {
        local $ctx->{ __stash }->{ vars }->{ __first__ } = 1 if ( $i == 0 );
        local $ctx->{ __stash }{ entry }   = $entry;
        local $ctx->{ __stash }{ blog }    = $entry->blog;
        local $ctx->{ __stash }{ blog_id } = $entry->blog_id;
        local $ctx->{ __stash }->{ vars }->{ __counter__ } = $i + 1;
        local $ctx->{ __stash }->{ vars }->{ __odd__ } = $odd;
        local $ctx->{ __stash }->{ vars }->{ __even__ } = $even;
        local $ctx->{ __stash }->{ vars }->{ __last__ } = 1 if ( !defined( $entries[ $i + 1 ] ) );
        my $out = $builder->build( $ctx, $tokens, $cond );
        if ( !defined( $out ) ) { return $ctx->error( $builder->errstr ) };
        $res .= $out;
        if ( $odd == 1 ) { $odd = 0 } else { $odd = 1 };
        if ( $even == 1 ) { $even = 0 } else { $even = 1 };
        $i++;
    }
    $res;
}

sub _hdlr_query {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $key = $args->{ key };
    my $val = $app->param( $key );
    return '' unless $val;
    $val = to_utf8( $val );
    $val = utf8_on( $val );
    return $val;
}

sub _hdlr_user_agent {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $agent = $app->get_header( 'User-Agent' );
    return '' unless $agent;
    if ( $args->{ career } ) {
        if ( $agent =~ /DoCoMo/ ) {
            return 'DoCoMo';
        } elsif ( $agent =~ /UP\.Browser/ ) {
            return 'AU';
        } elsif ( $agent =~ /SoftBank/ ) {
            return 'SoftBank';
        } elsif ( $agent =~ /Vodafone/ ) {
            return 'SoftBank';
        } elsif ( $agent =~ /iPhone/ ) {
            return 'iPhone';
        } elsif ( $agent =~ /iPod/ ) {
            return 'iPhone';
        } elsif ( $agent =~ /Mozilla/ ) {
            return 'PC';
        } elsif ( $agent =~ /Opera/ ) {
            return 'PC';
        } else {
            return 'Other';
        }
    } else {
        return $agent;
    }
}

sub _hdlr_if_login {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $user = get_user( $app );
    return 1 if defined $user;
    return 0;
}

sub _hdlr_if_keitai {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    return is_ua_keitai( $app );
}

sub _hdlr_if_iphone {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    return is_ua_iPhone( $app );
}

sub _hdlr_plugin_path {
    my ( $ctx, $args, $cond ) = @_;
    my $component = $args->{ component };
    my $folder = $args->{ folder };
    my @option = split( /,/, $folder );
    if ( $component ) {
        my $component = MT->component( $component );
        if ( defined $component ) {
            if ( @option ) {
                return add_slash( File::Spec->catdir( $component->path, @option ) );
            } else {
                return add_slash( $component->path );
            }
        }
    }
    return '';
}

sub _hdlr_pass_tokens {
    my ( $ctx, $args, $cond ) = @_;
    $ctx->stash( 'builder' )->build( $ctx, $ctx->stash( 'tokens' ), $cond );
}

sub _hdlr_blog_files_match_directive {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    my $exclude_extension = $blog->exclude_extension || 'php,cgi,fcgi';
    $exclude_extension =~ s/\s//g;
    my $lc = lc ( $exclude_extension );
    my $uc = uc ( $exclude_extension );
    my @extensions = split( /,/, $lc );
    my @extensions_uc = split( /,/, $uc );
    push ( @extensions, @extensions_uc );
    $exclude_extension = join( '|', @extensions );
    my $FilesMatch = '<FilesMatch .*\.(?!' . $exclude_extension . ')>';
    return $FilesMatch;
}

sub _hdlr_blog_files_directive {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    my $dynamic_extension = $blog->dynamic_extension || 'mtml';
    $dynamic_extension =~ s/\s//g;
    my $lc = lc ( $dynamic_extension );
    my $uc = uc ( $dynamic_extension );
    my @extensions = split( /,/, $lc );
    my @extensions_uc = split( /,/, $uc );
    push ( @extensions, @extensions_uc );
    $dynamic_extension = join( '|', @extensions );
    my $Files = '<Files ~ "\.(' . $dynamic_extension . ')$">';
    return $Files;
}

sub _hdlr_dynamic_cache {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    if ( $blog->dynamic_cache ) {
        return '$mt->caching( true );';
    }
    return '';
}

sub _hdlr_dynamic_conditional {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    if ( $blog->dynamic_conditional ) {
        return '$mt->conditional( true );';
    }
    return '';
}

sub _hdlr_dynamic_search_cache {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    if ( $blog->search_cache ) {
        return '1';
    }
    return '0';
}

sub _hdlr_dynamic_search_conditional {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    if ( $blog->search_conditional ) {
        return '$mt->conditional( true );';
    }
    return '';
}

sub _hdlr_dynamic_search_cache_expiration {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    if ( $blog->search_cache_expiration ) {
        return $blog->search_cache_expiration;
    }
    return '7200';
}

sub _hdlr_dynamic_directory_index {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    if ( $blog->index_files ) {
        return $blog->index_files;
    }
    return 'index.mtml';
}

sub _hdlr_dynamic_extension {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    if ( $blog->dynamic_extension ) {
        return $blog->dynamic_extension;
    }
    return 'mtml';
}

sub _hdlr_dynamic_exclude_extension {
    my ( $ctx, $args, $cond ) = @_;
    my $blog = $ctx->stash( 'blog' );
    if ( $blog->exclude_extension ) {
        return $blog->exclude_extension;
    }
    return 'php,cgi,fcgi';
}

sub _filter_build_mtml {
    my ( $str, $arg, $ctx ) = @_;
    my $builder = $ctx->stash( 'builder' );
    my $tokens = $builder->compile( $ctx, $str );
    return $ctx->error( $builder->errstr ) unless defined $tokens;
    my $out = $builder->build( $ctx, $tokens );
    return $ctx->error( $builder->errstr ) unless defined $out;
    return $out;
}

sub _cfg_prefs_param {
    my ( $cb, $app, $param, $tmpl ) = @_;
    my $pointer_node = $tmpl->getElementById( 'dynamic_publishing_options' );
    return unless $pointer_node;
    $param->{ dynamic_enabled } = 1;
    $param->{ hide_build_option } = 0;
    $param->{ dynamic_caching } = $param->{ dynamic_cache };
    my $inner =<<'MTML';
    <__trans_section component="DynamicMTML">
    <ul>
        <li>
            <input type="checkbox" id="search_cache" name="search_cache" <mt:if name="search_cache">checked="checked"</mt:if> value="1" />
            <label for="search_cache"><__trans phrase="Enable dynamic search cache"></label>
            ( <label for="search_cache_expiration"><__trans phrase="Cache expiration"></label>
            <input
                type="text" name="search_cache_expiration" id="search_cache_expiration" style="width:100px"
                value="<$mt:var name="search_cache_expiration" escape="html"$>" mt:watch-change="1" /> )
        </li>
        <li>
            <input type="checkbox" id="search_conditional" name="search_conditional" <mt:if name="search_conditional">checked="checked"</mt:if> value="1" />
            <label for="search_conditional"><__trans phrase="Enable search conditional retrieval"></label>
        </li>
    </ul>
    <input type="hidden" name="dynamic_cache" value="0" />
    <input type="hidden" name="dynamic_conditional" value="0" />
    <input type="hidden" name="search_cache" value="0" />
    <input type="hidden" name="search_conditional" value="0" />
    </__trans_section>
MTML
    my $dynamic_search_options_node = $tmpl->createElement( 'app:setting', {
        id => 'dynamic_search_options',
        label => $plugin->translate( 'Dynamic Search Options' ),
        show_label => 0,
    } );
    $dynamic_search_options_node->innerHTML( $inner );
    $tmpl->insertAfter( $dynamic_search_options_node, $pointer_node );
    $inner =<<'MTML';
        <input type="text" name="dynamic_extension" id="dynamic_extension" class="full-width" value="<mt:var name="dynamic_extension">" size="30" />
MTML
    my $dynamic_extension_node = $tmpl->createElement( 'app:setting', {
        id => 'dynamic_extension',
        label => $plugin->translate( 'Dynamic Extensions' ),
        show_label => 1,
    } );
    $dynamic_extension_node->innerHTML( $inner );
    $tmpl->insertAfter( $dynamic_extension_node, $dynamic_search_options_node );
    $inner =<<'MTML';
        <input type="text" name="exclude_extension" id="exclude_extension" class="full-width" value="<mt:var name="exclude_extension">" size="30" />
MTML
    my $exclude_extension_node = $tmpl->createElement( 'app:setting', {
        id => 'exclude_extension',
        label => $plugin->translate( 'Exclude Extensions' ),
        show_label => 1,
    } );
    $exclude_extension_node->innerHTML( $inner );
    $tmpl->insertAfter( $exclude_extension_node, $dynamic_extension_node );
    $inner =<<'MTML';
        <input type="text" name="index_files" id="index_files" class="full-width" value="<mt:var name="index_files">" size="30" />
MTML
    my $index_files_node = $tmpl->createElement( 'app:setting', {
        id => 'index_files',
        label => $plugin->translate( 'Directory Index' ),
        show_label => 1,
    } );
    $index_files_node->innerHTML( $inner );
    $tmpl->insertAfter( $index_files_node, $exclude_extension_node );
}

sub _cfg_prefs_source {
    my ( $cb, $app, $tmpl ) = @_;
    $$tmpl =~ s/(id="dynamic_cache")/$1 value="1"/;
    $$tmpl =~ s/(id="dynamic_conditional")/$1 value="1"/;
}

sub _list_template_source {
    my ( $cb, $app, $tmpl ) = @_;
    my $old = quotemeta( '<li><a href="<mt:var name="script_url">?__mode=list_template&amp;filter_key=backup_templates&amp;blog_id=<mt:var name="blog_id">" class="icon-left icon-related"><__trans phrase="Template Backups"></a></li>' );
    my $blog_id;
    my $return_args = '__mode=list_template&amp;blog_id=';
    if ( $app->blog ) {
        $blog_id = $app->blog->id;
        $return_args .= $blog_id;
    }
    my $link = $app->base . $app->uri(
                    mode => 'install_dynamic_mtml',
                    args => { blog_id => $blog_id,
                              return_args => $return_args } );
    my $label = $plugin->translate( 'Install DynamicMTML' );
    my $new = "<li><a class=\"icon-left icon-related\" href=\"$link\">$label</a><li>" if $app->blog;
    my $link2 = $app->base . $app->uri(
                    mode => 'flush_dynamic_cache',
                    args => { blog_id => $blog_id,
                              return_args => $return_args } );
    my $label2 = $plugin->translate( 'Flush Dynamic Cache' );
    my $new2 = "<li><a class=\"icon-left icon-related\" href=\"$link2\">$label2</a><li>";
    $$tmpl =~ s/($old)/$new$new2$1/;
}

sub _list_template_param {
    my ( $cb, $app, $param, $tmpl ) = @_;
    my $pointer_node = $tmpl->getElementById( 'saved-settings' );
    if ( $app->param( 'installed_dynamicmtml' ) ) {
        $param->{ saved } = 1;
        $pointer_node->innerHTML( $plugin->translate( 'Install DynamicMTML was successful.' ) );
    } elsif ( $app->param( 'not_installed_dynamicmtml' ) ) {
        $param->{ saved } = 1;
        $pointer_node->innerHTML( $plugin->translate( 'Install DynamicMTML failed.' ) );
    } elsif ( $app->param( 'flush_dynamic_cache' ) ) {
        $param->{ saved } = 1;
        $pointer_node->innerHTML( $plugin->translate( 'Flush Dynamic Cache was successful.' ) );
    } elsif ( $app->param( 'not_flush_dynamic_cache' ) ) {
        $param->{ saved } = 1;
        $pointer_node->innerHTML( $plugin->translate( 'Cache file was not found.' ) );
    } elsif ( $app->param( 'no_cache_directory' ) ) {
        $param->{ saved } = 1;
        $pointer_node->innerHTML( $plugin->translate( 'Files for PowerCMS directory unexists. Please make directory [_1], and give enough permission to write from web server.',
                                                      powercms_files_dir_path() ) );
    }
}

1;